class RadsController < ApplicationController
  before_filter :login_required
  skip_before_filter :login_required, :login_from_cookie, :only => [ :click_to, :image, :view_banner ]

  # Redirect to the URL while counting the click-thru
  def click_to
    redirect_to rads_url(params[:id])
  end

  # Display the advertisement image from the database
  #  TODO: Sends all data as jpeg, but doesn't check if it really is a jpeg.
  def image
    @image_data = RadsBanner.find( params[:id] ).image
    send_data(@image_data, :type => 'image/jpeg', :filename => 'image.jpg', :disposition => 'inline')
  end
  
  def view_banner
    @banner_id = params[:id]
  end
  
  # Delete banner (misnomer) if it's yours. dependent destroy!
  def delete_banner
    if current_user.administrator || (not RadsCampaign.find_by_id_and_user_id(params[:campaign_id], current_user.id).nil?)
      RadsBanner.destroy(params[:id])
    end
    redirect_to '/rads/admin/' + params[:campaign_id]
  end

  # Delete campaign (misnomer) if it's yours. dependent destroy!
  def delete_campaign
    if current_user.administrator || (not RadsCampaign.find_by_id_and_user_id(params[:id], current_user.id).nil?)
      RadsCampaign.destroy(params[:id])
    end
    redirect_to '/rads/admin/'
  end

  # Administer RADS, depending on login.
  def admin
    @campaign = RadsCampaign.find_by_id(params[:id])
    @campaign = RadsCampaign.find_by_id(params[:campaign][:id]) if @campaign.nil? and not params[:campaign].nil?
    @authorized = current_user.administrator
    if current_user.administrator
      @campaigns = RadsCampaign.all
      @campaign = RadsCampaign.find_by_id(params[:id])
      @campaign = RadsCampaign.find_by_id(params[:campaign][:id]) if @campaign.nil? and not params[:campaign].nil?
    else
      @campaigns = RadsCampaign.all(:conditions => "user_id = #{current_user.id}")
      @campaign = RadsCampaign.find_by_id_and_user_id(params[:id], current_user.id)
      @campaign = RadsCampaign.find_by_id_and_user_id(params[:campaign][:id], current_user.id) if @campaign.nil? and not params[:campaign].nil?
    end
  end

  # Create a banner; needs to be in a campaign you own, or you need to be administrator.
  def create_banner
    if current_user.administrator || (not RadsCampaign.find_by_id_and_user_id(params[:campaign_id], current_user.id).nil?)
      RadsBanner.create(:name => params[:name], :uploaded_file => params[:uploaded_file], :rads_banner_type_id => params[:type_id], :url => params[:url], :rads_campaign_id => params[:campaign_id])
    end
    redirect_to '/rads/admin/' + params[:campaign_id]
  end

  # Create a campaign; administrator can create campaigns for other users.
  def create_campaign
    user_id = current_user.administrator ? params[:user_id] : current_user.id
    campaign = RadsCampaign.create(:name => params[:name], :budget => params[:budget], :user_id => user_id )
    redirect_to '/rads/admin/' + campaign.id.to_s
  end

  protected
      # Count the click and return the URL
      def rads_url(banner_id)
        banner = RadsBanner.find(banner_id)
        banner.rads_banner_stats[0].increment! :clicks
      return banner.url
    end

    # User needs to be administrator or rads user
    def authorized?(action = action_name, resource = nil)
      logged_in? && ( current_user.is_administrator? || current_user.is_radsuser? )
    end


end
