# This controller handles the login/logout function of the site.  
class SessionsController < ApplicationController
  # this just to satisfy new_session_path
  def new
    redirect_to '/'
  end
  #end

  def create
    logout_keeping_session!
    user = User.authenticate(params[:login], params[:password])
    if user
      # Protects against session fixation attacks, causes request forgery
      # protection if user resubmits an earlier form using back
      # button. Uncomment if you understand the tradeoffs.
      # reset_session
      self.current_user = user
      new_cookie_flag = (params[:remember_me] == "1")
      handle_remember_cookie! new_cookie_flag
      redirect_back_or_default('/')
      flash[:notice] = "Logged in successfully"
    else
      note_failed_signin
      @login       = params[:login]
      @remember_me = params[:remember_me]
      redirect_to '/'
    end
  end

  def destroy
    was_logged_in = logged_in?
    logout_killing_session!
    # TODO: remove all session vars except session_id and _csrf_token
    session.delete(:fforum_return_uri)
    session.delete(:fforum_root_id)
    flash[:notice] = "You have been logged out." if was_logged_in
    redirect_to '/'
  end

protected
  # Track failed login attempts
  def note_failed_signin
    flash[:error] = "Couldn't log you in as '#{params[:login]}'"
    logger.warn "Failed login for '#{params[:login]}' from #{request.remote_ip} at #{Time.now.utc}"
  end
end
