class WordController < ApplicationController
  skip_before_filter :login_required, :login_from_cookie
  auto_complete_for :word, :word
  def method_missing(word)
    # return 404 if more the one word
    if word.split.length > 1
      flash[:error] = "Please only ONE word..."
      return redirect_to '/' if word.split.length > 1
    end
    # show the word if it exists
    @word = Word.find_by_word(word)
    if not @word.nil?
      @word.increment_views
      return render :action => "index"
    end
    # create the word if logged_in
    return redirect_to :action => :create_word, :word => word if logged_in?
    # notify if not logged_in
    flash[:error] = "Unkown word: <strong>" + CGI.escapeHTML(word) + "</strong>. You need to be logged in to add it!"
    return redirect_to '/'
  end
  def find_word
    @word = Word.random
    return render :action => "index", :status => :not_found if not params[:word]
    return render :action => "index", :status => :not_found if not params[:word][:word]
    if params[:word][:word].split.length > 1
      flash[:error]='One word at a time please...'
      flash[:too_many_words] = true
      return render :action => "index", :word => params[:word][:word]
    end
    redirect_to :action => params[:word][:word]
  end
  def not_found
    @word = Word.random
    render :action => "index", :status => :not_found
  end
  def redirect
    redirect_to '/'
  end
  def random_word
    @word = Word.random
    render :action => "index"
  end
  def delete_word
    word = Word.find(params[:id])
    word.deleted!
    word.judged_by=current_user
    word.judged_at=Time.now()
    word.save
    redirect_to :action => word.word
  end
  def approve_word
    word = Word.find(params[:id])
    word.approved!
    word.judged_by=current_user
    word.judged_at=Time.now()
    word.save
    redirect_to :action => word.word
  end
  def save_word
    word = params[:word]
    @word = Word.find(params[:id])
    u = @word.updated_at
    if @word.update_attributes(word)
      flash[:saved] = true if @word.updated_at.to_i > u.to_i
    else
      flash[:error] = "NOT SAVED"
    end
    redirect_to :action => @word.word
  end

  # vote_fu
  def _vote_for
    @word = Word.find(params[:id])
    current_user.vote_for(@word)
    render :partial => "partials/votes"
  end

  def _vote_against
    @word = Word.find(params[:id])
    current_user.vote_against(@word)
    render :partial => "partials/votes"
  end

  def _vote_reset
    @word = Word.find(params[:id])
    current_user.vote_reset(@word)
    render :partial => "partials/votes"
  end

  def create_word
    word = params[:word]
    new_word = Word.new( :word => word,
      :created_by => current_user,
      :updated_by => current_user,
      :pap_description => "No description yet"
    );
    if new_word.save
      # create the forum
      FforumPost.create(:forum_type=>'Word',
        :created_by => current_user,
        :updated_by => current_user,
        :forum_type_id => new_word.id,
        :title => new_word.word,
        :content => 'dummy' )
      flash[:notice] = 'You created a new word!'
      redirect_to :action => new_word.word
    else
      flash[:error] = "<strong>" + CGI.escapeHTML(word) + "</strong> is not a valid word."
      redirect_to '/'
    end
  end
end
