class RadsCampaign < ActiveRecord::Base
  has_many :rads_banners, :dependent => :destroy
  belongs_to :user
  
  def banner(banner_type_id)
    banner = self.rads_banners.find(:first, :order => 'RAND()', :conditions => ["rads_banner_type_id = ?", banner_type_id])
    if banner
      stats = banner.rads_banner_stats.find(:first, :conditions => ["MONTH(created_at) = ? and YEAR(created_at) = ?", Time.now.month, Time.now.year])
      stats = banner.rads_banner_stats.new if stats.nil?
      stats.impressions += 1
      stats.save
      return banner
    else
      return nil
    end
  end
  
  # Calculate budget used
  def budget_used
    budget_used = 0
    self.rads_banners.each{ |banner|
      stats = banner.rads_banner_stats.find(:first, :conditions => ["MONTH(created_at) = ? and YEAR(created_at) = ?", Time.now.month, Time.now.year])
      budget_used += stats.impressions * (banner.cpm.to_f / 1000.to_f) if stats
    }
    return budget_used
  end

  # Calculate remaining budget
  def budget_remaining
    return budget - budget_used
  end

  # Sum impressions from all banners in this campaign
  def total_impressions
    return 0 if rads_banner_ids.empty?
    RadsBannerStat.sum(:impressions, :conditions => "rads_banner_id IN (" + rads_banner_ids.join(',') + ")" )
  end

  # Sum clicks from all banners in this campaign
  def total_clicks
    return 0 if rads_banner_ids.empty?
    RadsBannerStat.sum(:clicks, :conditions => "rads_banner_id IN (" + self.rads_banner_ids.join(',') + ")" )
  end

  # Show the cost per click, and avoid division_by_zero
  def cost_per_click
    total_clicks > 0 ?  budget_used /  total_clicks : 0
  end

  def user_and_campaign_name
   user.name + ": " + name
  end
  
  def self.reloadable?
    false
  end
end
