class Word < ActiveRecord::Base
  acts_as_voteable
  serialize :proverbs
  validates_presence_of     :word
  validates_uniqueness_of   :word, :case_sensitive => false
  #validates_format_of :word, :with => /\A([a-z]['a-z]*[a-z])\Z/i
  validates_format_of :word, :with => /\A([\w]['\w]*[\w])\Z/i

  belongs_to :created_by, :class_name => 'User', :foreign_key => :created_by
  belongs_to :updated_by, :class_name => 'User', :foreign_key => :updated_by
  belongs_to :judged_by, :class_name => 'User', :foreign_key => :judged_by
  #file_column :picture

  # Compare two Word object by comparing their word attribute
  def <=>(other)
    self.word <=> other.word
  end

  # Is this word deleted?
  def deleted?
    return true if self.status == 3
  end

  # Is this word approved?
  def approved?
    return true if self.status == 2
  end

  def undecided?
    return true if self.status == 1
  end

  def deleted!
    self.status = 3
  end

  def approved!
    self.status = 2
  end

  def undecided!
    self.status = 1
  end


  # Increment the view counter for a word, without changing updated_at (see http://blog.evanweaver.com/articles/2006/12/26/hacking-activerecords-automatic-timestamps/)
  def increment_views
    class << self
      def record_timestamps
        false
      end
    end
    self.increment! :views
    class << self
      remove_method :record_timestamps
    end
  end

  def self.random
    offset = rand(Word.count :all)
    Word.find :first, :offset => offset
  end

  def self.recently_added(status)
    s = case status
          when :approved then 2
          when :deleted then 3
          else 1
        end
    o = case status
          when :approved then 'judged_at'
          when :deleted then 'judged_at'
          else 'created_at'
        end
    Word.find :all, :limit => 10, :order => o + ' DESC', :conditions => 'status = ' + s.to_s
  end

  def self.count_approved
    Word.count(:conditions => "status = 2")
  end

  def self.count_undecided
    Word.count(:conditions => "status < 2")
  end


  def self.most_viewed
    Word.find :all, :limit => 10, :order => 'views DESC, word'
  end

  def self.recently_updated
    Word.find :all, :limit => 5, :order => 'updated_at DESC', :conditions => 'status = 2'
  end

  def previous_approved_words(n)
    Word.find(:all, :limit => n, :order => 'word DESC', :conditions => [ "word < ? AND status = 2 ", self.word ])
  end

  def next_approved_words(n)
    Word.find(:all, :limit => n, :order => 'word ASC', :conditions => [ "word > ? AND status = 2 ", self.word ])
  end
  
  def list_proverbs
    Proverb.find(self.proverbs) if not self.proverbs.nil?
  end
  def status_s
    case self.status
          when 2 then 'approved'
          when 3 then 'deleted'
          else 'undecided'
        end
  end

end
